from sawsi.aws import shared
from sawsi.aws.sns import wrapper


class SNSAPI:
    """
    이메일 전송 API
    """
    def __init__(self, credentials=None, region=shared.DEFAULT_REGION):
        """
        :param credentials: {
            "aws_access_key_id": "str",
            "aws_secret_access_key": "str",
            "region_name": "str",
            "profile_name": "str",
        }
        """
        self.boto3_session = shared.get_boto_session(credentials)
        self.region_name = region
        self.sns = wrapper.SNS(self.boto3_session, region)

    def send_message(self, topic_arn, message):
        response = self.sns.send_message_to_topic(topic_arn, message)
        return response


if __name__ == '__main__':
    pass



import boto3
from sawsi.aws import shared
from typing import List, Literal, Optional


class CodeBuildAPI:
    def __init__(self, credentials=None, region=shared.DEFAULT_REGION):
        """
        :param credentials: {
            "aws_access_key_id": "str",
            "aws_secret_access_key": "str",
            "region_name": "str",
            "profile_name": "str",
        }
        """
        self.boto3_session = shared.get_boto_session(credentials)
        self.cache = {}
        self.client = boto3.client('codebuild', region_name=region)

    def batch_get_builds(self, build_ids: List[str]):
        # 커밋 기반으로 파일 가쟈오기
        response = self.client.batch_get_builds(
            ids=build_ids  # 여기에 조회하고자 하는 빌드 ID를 리스트 형태로 입력
        )
        return response

    def list_builds_for_project(self, project_name: str, sort_order: Literal["DESCENDING", "ASCENDING"] = 'DESCENDING', next_token: Optional[str] = None):
        # 빌드 정보 목록 조회
        payload = {
            'projectName': project_name,
            'sortOrder': sort_order,
        }
        if next_token:
            payload['nextToken'] = next_token
        response = self.client.list_builds_for_project(**payload)
        return response


if __name__ == '__main__':
    pass


"""
각 함수에 데코레이터를 적용하여 특정 경로로 라우팅되도록 만들기 위해 데코레이터를 정의합니다.
"""

controller_registry = {}


def controller(func):
    # 함수가 속한 모듈의 경로와 함수 이름을 합쳐서 full_module_name을 생성
    module_name = func.__module__  # 함수가 속한 모듈 경로를 자동으로 가져옴
    func_name = func.__name__  # 함수 이름
    full_module_name = f"{module_name}.{func_name}"

    # controller_registry에 등록
    controller_registry[full_module_name] = func

    return func



import traceback
import datetime
from croniter import croniter
from concurrent.futures import ThreadPoolExecutor


pool = ThreadPoolExecutor(max_workers=8)
futures = []


def join():
    for future in futures:
        future.result()


def try_run(func, *args, **kwargs):
    return func(*args, **kwargs)


class ScheduledTask:
    def __init__(self, cron_expression):
        self.cron_expression = cron_expression

    def __enter__(self):
        now = datetime.datetime.now() + datetime.timedelta(seconds=5)
        iter = croniter(self.cron_expression, now)
        prev_schedule = iter.get_prev(datetime.datetime)

        if not (prev_schedule.year == now.year and prev_schedule.month == now.month and
                prev_schedule.day == now.day and prev_schedule.hour == now.hour and
                prev_schedule.minute == now.minute):
            def run(*args, **kwargs):
                class Empty:
                    @classmethod
                    def result(cls):
                        pass
                return Empty
            return run

        def run(*args, **kwargs):
            global futures, pool
            future = pool.submit(try_run, *args, **kwargs)
            futures.append(future)
            return future
        return run

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_type:
            traceback_str = ''.join(traceback.format_exception(exc_type, exc_val, exc_tb))
            self.handle_error(exc_val, traceback_str)
            return True  # 에러를 처리하고 정상 종료

    def handle_error(self, exc_val, exc_tb):
        # 에러 처리 로직
        message = f'Error on scheduler: {exc_val}, {exc_tb}'
        raise RuntimeError(message)



if __name__ == '__main__':
    raise RuntimeError('pk')